@@ -3,14 +3,6 @@ from concurrent.futures import ProcessPoolExecutor, process
 
 from rich import pretty, traceback
 
-from .config import config
-
-# version
-from .version import __version__
-
-# updater
-from .updater import Updater
-
 # grid
 from .components import Grid, Coords, GridSpec, UniformGrid, CustomGrid, AutoGrid
 
@@ -34,16 +26,28 @@ from .components import GaussianBeam, AstigmaticGaussianBeam
 
 # monitors
 from .components import FieldMonitor, FieldTimeMonitor, FluxMonitor, FluxTimeMonitor
-from .components import ModeMonitor, ModeFieldMonitor, PermittivityMonitor
+from .components import ModeMonitor, ModeSolverMonitor, PermittivityMonitor
+from .components import Near2FarAngleMonitor, Near2FarCartesianMonitor, Near2FarKSpaceMonitor
+from .components.monitor import DiffractionMonitor
 
 # simulation
 from .components import Simulation
 
 # data
-from .components import SimulationData, FieldData, FluxData, FluxTimeData
-from .components import DATA_TYPE_MAP, ScalarFieldData, ScalarFieldTimeData
-from .components import ModeData, ModeAmpsData, ModeIndexData, ModeFieldData, PermittivityData
-from .components import ScalarPermittivityData
+from .components import ScalarFieldDataArray, ScalarModeFieldDataArray, ScalarFieldTimeDataArray
+from .components import ModeAmpsDataArray, ModeIndexDataArray
+from .components import FluxDataArray, FluxTimeDataArray
+from .components import Near2FarAngleDataArray, Near2FarCartesianDataArray, Near2FarKSpaceDataArray
+from .components.data.data_array import DiffractionDataArray
+from .components import FieldData, FieldTimeData, PermittivityData
+from .components import FluxData, FluxTimeData
+from .components import ModeData, ModeSolverData
+from .components import AbstractNear2FarData
+from .components import Near2FarAngleData, Near2FarCartesianData, Near2FarKSpaceData
+from .components import Near2FarSurface, RadiationVectors
+from .components.data.monitor_data import DiffractionData
+from .components import SimulationData
+from .components import DATA_TYPE_MAP
 
 # boundary
 from .components import BoundarySpec, Boundary, BoundaryEdge, BoundaryEdgeType
@@ -54,9 +58,6 @@ from .components import DefaultPMLParameters, DefaultStablePMLParameters, Defaul
 # constants imported as `C_0 = td.C_0` or `td.constants.C_0`
 from .constants import C_0, ETA_0, HBAR, EPSILON_0, MU_0, Q_e, inf
 
-# plugins typically imported as `from tidy3d.plugins import DispersionFitter`
-from . import plugins
-
 # material library dict imported as `from tidy3d import material_library`
 # get material `mat` and variant `var` as `material_library[mat][var]`
 from .material_library import material_library
@@ -69,7 +70,16 @@ from .components.monitor import Monitor
 from .components.grid import YeeGrid, FieldGrid, Coords1D
 
 # logging
-from .log import log, set_logging_file
+from .log import log, set_logging_file, WebError
+
+# config
+from .config import config
+
+# version
+from .version import __version__
+
+# updater
+from .updater import Updater
 
 
 def set_logging_level(level: str) -> None:
@@ -82,4 +92,5 @@ def set_logging_level(level: str) -> None:
 
 # make all stdout and errors pretty
 pretty.install()
+log.info(f"Using client version: {__version__}")
 # traceback.install()
