@@ -69,7 +69,7 @@ class Tidy3dBaseModel(pydantic.BaseModel):
         }
         frozen = True
         allow_mutation = False
-        copy_on_model_validation = False
+        copy_on_model_validation = "none"
 
     _cached_properties = pydantic.PrivateAttr({})
 
