@@ -121,10 +121,14 @@ class BlochBoundary(BoundaryEdge):
         angle_theta = source.angle_theta
         angle_phi = source.angle_phi
 
+        # handle the special case to avoid tiny numerical fluctuations
+        if angle_theta == 0:
+            return cls(bloch_vec=0)
+
         if source.direction == "-":
             angle_theta += np.pi
-            angle_phi += np.pi
-            kmag *= -1.0
+            # angle_phi += np.pi
+            # kmag *= -1.0
 
         k_local = [
             kmag * np.sin(angle_theta) * np.cos(angle_phi),
@@ -595,27 +599,6 @@ class BoundarySpec(Tidy3dBaseModel):
             return self.z
         raise DataError(f"field_name '{field_name}' not found")
 
-    def __setitem__(self, field_name: str, field: Boundary):
-        """Set the :class:`Boundary` field by name (``boundary_spec[field_name]=field``).
-
-        Parameters
-        ----------
-        field_name : ``str``
-            Name of the axis, eg. "y" along which :class:`Boundary` is to be set.
-        field : :class:`Boundary`
-            Boundary condition to set along the named axis.
-        """
-        if field_name == "x":
-            self.x = field
-            return
-        if field_name == "y":
-            self.y = field
-            return
-        if field_name == "z":
-            self.z = field
-            return
-        raise DataError(f"field_name '{field_name}' not found")
-
     @classmethod
     def pml(cls, x: bool = False, y: bool = False, z: bool = False):
         """Default PML along specified directions
