@@ -1,4 +1,5 @@
 """ Defines classes specifying meshing in 1D and a collective class for 3D """
+from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from typing import Tuple, List, Union
@@ -128,9 +129,7 @@ class GridSpec1d(Tidy3dBaseModel, ABC):
         last_step = bounds[-1] - bounds[-2]
         add_left = bounds[0] - first_step * np.arange(num_layers[0], 0, -1)
         add_right = bounds[-1] + last_step * np.arange(1, num_layers[1] + 1)
-        new_bounds = np.concatenate((add_left, bounds, add_right))
-
-        return new_bounds
+        return np.concatenate((add_left, bounds, add_right))
 
 
 class UniformGrid(GridSpec1d):
@@ -183,10 +182,7 @@ class UniformGrid(GridSpec1d):
         # Adjust step size to fit simulation size exactly
         dl_snapped = size / num_cells if size > 0 else self.dl
 
-        # Make bounds
-        bound_coords = center - size / 2 + np.arange(num_cells + 1) * dl_snapped
-
-        return bound_coords
+        return center - size / 2 + np.arange(num_cells + 1) * dl_snapped
 
 
 class CustomGrid(GridSpec1d):
@@ -198,7 +194,7 @@ class CustomGrid(GridSpec1d):
     >>> grid_1d = CustomGrid(dl=[0.2, 0.2, 0.1, 0.1, 0.1, 0.2, 0.2])
     """
 
-    dl: List[pd.PositiveFloat] = pd.Field(
+    dl: Tuple[pd.PositiveFloat, ...] = pd.Field(
         ...,
         title="Customized grid sizes.",
         description="An array of custom nonuniform grid sizes. The resulting grid is centered on "
@@ -408,10 +404,10 @@ class GridSpec(Tidy3dBaseModel):
         units=MICROMETER,
     )
 
-    override_structures: List[Structure] = pd.Field(
-        [],
+    override_structures: Tuple[Structure, ...] = pd.Field(
+        (),
         title="Grid specification override structures",
-        description="A list of structures that is added on top of the simulation structures in "
+        description="A set of structures that is added on top of the simulation structures in "
         "the process of generating the grid. This can be used to refine the grid or make it "
         "coarser depending than the expected need for higher/lower resolution regions.",
     )
@@ -422,26 +418,28 @@ class GridSpec(Tidy3dBaseModel):
         grid_list = [self.grid_x, self.grid_y, self.grid_z]
         return np.any([isinstance(mesh, AutoGrid) for mesh in grid_list])
 
-    def wavelength_from_sources(self, sources: List[SourceType]) -> pd.PositiveFloat:
+    @staticmethod
+    def wavelength_from_sources(sources: List[SourceType]) -> pd.PositiveFloat:
         """Define a wavelength based on supplied sources. Called if auto mesh is used and
         ``self.wavelength is None``."""
 
-        # Use central frequency of sources, if any.
-        freqs = np.array([source.source_time.freq0 for source in sources])
         # no sources
-        if len(freqs) == 0:
+        if len(sources) == 0:
             raise SetupError(
                 "Automatic grid generation requires the input of 'wavelength' or sources."
             )
+
+        # Use central frequency of sources, if any.
+        freqs = np.array([source.source_time.freq0 for source in sources])
+
         # multiple sources of different central frequencies
-        if len(freqs) > 0 and not np.all(np.isclose(freqs, freqs[0])):
+        if not np.all(np.isclose(freqs, freqs[0])):
             raise SetupError(
                 "Sources of different central frequencies are supplied. "
                 "Please supply a 'wavelength' value for 'grid_spec'."
             )
 
-        wavelength = C_0 / freqs[0]
-        return wavelength
+        return C_0 / freqs[0]
 
     def make_grid(
         self,
@@ -473,27 +471,27 @@ class GridSpec(Tidy3dBaseModel):
 
         # Set up wavelength for automatic mesh generation if needed.
         wavelength = self.wavelength
-        if self.wavelength is None and self.auto_grid_used:
+        if wavelength is None and self.auto_grid_used:
             wavelength = self.wavelength_from_sources(sources)
             log.info(f"Auto meshing using wavelength {wavelength:1.4f} defined from sources.")
 
         coords_x = self.grid_x.make_coords(
             axis=0,
-            structures=structures + self.override_structures,
+            structures=list(structures) + list(self.override_structures),
             symmetry=symmetry,
             wavelength=wavelength,
             num_pml_layers=num_pml_layers[0],
         )
         coords_y = self.grid_y.make_coords(
             axis=1,
-            structures=structures + self.override_structures,
+            structures=list(structures) + list(self.override_structures),
             symmetry=symmetry,
             wavelength=wavelength,
             num_pml_layers=num_pml_layers[1],
         )
         coords_z = self.grid_z.make_coords(
             axis=2,
-            structures=structures + self.override_structures,
+            structures=list(structures) + list(self.override_structures),
             symmetry=symmetry,
             wavelength=wavelength,
             num_pml_layers=num_pml_layers[2],
@@ -510,7 +508,7 @@ class GridSpec(Tidy3dBaseModel):
         max_scale: pd.PositiveFloat = 1.4,
         override_structures: List[Structure] = (),
         mesher: MesherType = GradedMesher(),
-    ) -> "GridSpec":
+    ) -> GridSpec:
         """Use the same :class:`AutoGrid` along each of the three directions.
 
         Parameters
@@ -546,7 +544,7 @@ class GridSpec(Tidy3dBaseModel):
         )
 
     @classmethod
-    def uniform(cls, dl: float) -> "GridSpec":
+    def uniform(cls, dl: float) -> GridSpec:
         """Use the same :class:`UniformGrid` along each of the three directions.
 
         Parameters
