@@ -7,27 +7,17 @@ from math import isclose
 
 import pydantic as pd
 import numpy as np
-from scipy.optimize import root_scalar
+from pyroots import Brentq
 from shapely.strtree import STRtree
-from shapely.geometry import Polygon
+from shapely.geometry import box as shapely_box
 
 from ..base import Tidy3dBaseModel
-from ..types import Axis, Array
+from ..types import Axis, ArrayLike
 from ..structure import Structure
 from ...log import SetupError, ValidationError
 from ...constants import C_0, fp_eps
 
-
-# pylint:disable=abstract-method
-class IndexedBox(Polygon):
-    """A 2D shapely box with an extra ``str_ind`` variable to keep structure indexes."""
-
-    def __init__(self, minx, miny, maxx, maxy):
-        """Initialize with the same arguments as ``shapely.geometry.box``."""
-        coords = [(maxx, miny), (maxx, maxy), (minx, maxy), (minx, miny)]
-        super().__init__(coords)
-
-        self.str_ind = None
+_ROOTS_TOL = 1e-10
 
 
 class Mesher(Tidy3dBaseModel, ABC):
@@ -40,17 +30,17 @@ class Mesher(Tidy3dBaseModel, ABC):
         structures: List[Structure],
         wavelength: pd.PositiveFloat,
         min_steps_per_wvl: pd.NonNegativeInt,
-    ) -> Tuple[Array[float], Array[float]]:
+    ) -> Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]:
         """Calculate the positions of all bounding box interfaces along a given axis."""
 
     @abstractmethod
     def make_grid_multiple_intervals(
         self,
-        max_dl_list: Array[float],
-        len_interval_list: Array[float],
+        max_dl_list: ArrayLike[float, 1],
+        len_interval_list: ArrayLike[float, 1],
         max_scale: float,
         is_periodic: bool,
-    ) -> List[Array[float]]:
+    ) -> List[ArrayLike[float, 1]]:
         """Create grid steps in multiple connecting intervals."""
 
 
@@ -65,7 +55,7 @@ class GradedMesher(Mesher):
         structures: List[Structure],
         wavelength: pd.PositiveFloat,
         min_steps_per_wvl: pd.NonNegativeInt,
-    ) -> Tuple[Array[float], Array[float]]:
+    ) -> Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]:
         """Calculate the positions of all bounding box interfaces along a given axis.
         In this implementation, in most cases the complexity should be O(len(structures)**2),
         although the worst-case complexity may approach O(len(structures)**3).
@@ -107,19 +97,15 @@ class GradedMesher(Mesher):
 
         # If empty simulation, return
         if len(structures) == 1:
-            return (domain_bounds, structure_steps)
+            interval_coords, max_steps = self.filter_min_step(domain_bounds, structure_steps)
+            return np.array(interval_coords), np.array(max_steps)
 
         # Bounding boxes with the meshing axis rotated to z
         struct_bbox = self.rotate_structure_bounds(structures, axis)
         # Rtree from the 2D part of the bounding boxes
         tree = self.bounds_2d_tree(struct_bbox)
 
-        intervals = {}
-        # Array of coordinates of all intervals; add the simulation domain bounds already
-        intervals["coords"] = list(domain_bounds)
-        # List of indexes of structures which are present in each interval
-        intervals["structs"] = [[]]  # will have len equal to len(intervals["coords"]) - 1
-
+        intervals = {"coords": list(domain_bounds), "structs": [[]]}
         # Iterate in reverse order as latter structures override earlier ones. To properly handle
         # containment then we need to populate interval coordinates starting from the top.
         # If a structure is found to be completely contained, the corresponding ``struct_bbox`` is
@@ -130,14 +116,16 @@ class GradedMesher(Mesher):
             if bbox is None:
                 # Structure has been removed because it is completely contained
                 continue
-            bbox_2d = IndexedBox(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
+            bbox_2d = shapely_box(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
 
             # List of structure indexes that may intersect the current structure in 2D
-            query_inds = [box.str_ind for box in tree.query(bbox_2d)]
+            query_inds = tree.query_items(bbox_2d)
 
             # Remove all lower structures that the current structure completely contains
-            inds_lower = [ind for ind in query_inds if ind < str_ind]
-            query_bbox = [struct_bbox[ind] for ind in inds_lower if struct_bbox[ind] is not None]
+            inds_lower = [
+                ind for ind in query_inds if ind < str_ind and struct_bbox[ind] is not None
+            ]
+            query_bbox = [struct_bbox[ind] for ind in inds_lower]
             bbox_contains_inds = self.contains_3d(bbox, query_bbox)
             for ind in bbox_contains_inds:
                 struct_bbox[inds_lower[ind]] = None
@@ -174,8 +162,8 @@ class GradedMesher(Mesher):
         self,
         intervals: Dict[str, List],
         str_ind: int,
-        str_bbox: Array[float],
-        bbox_contained_2d: List[Array[float]],
+        str_bbox: ArrayLike[float, 1],
+        bbox_contained_2d: List[ArrayLike[float, 1]],
         min_step: float,
     ) -> Dict[str, List]:
         """Figure out where to place the bounding box coordinates of current structure.
@@ -195,9 +183,9 @@ class GradedMesher(Mesher):
             of lists of structures contained in each interval.
         str_ind : int
             Index of the current structure.
-        str_bbox : Array[float]
+        str_bbox : ArrayLike[float, 1]
             Bounding box of the current structure.
-        bbox_contained_2d : List[Array[float]]
+        bbox_contained_2d : List[ArrayLike[float, 1]]
             List of 3D bounding boxes that contain the current structure in 2D.
         min_step : float
             Absolute minimum interval size to impose.
@@ -258,7 +246,7 @@ class GradedMesher(Mesher):
     @staticmethod
     def structure_steps(
         structures: List[Structure], wavelength: float, min_steps_per_wvl: float
-    ) -> Array[float]:
+    ) -> ArrayLike[float, 1]:
         """Get the minimum mesh required in each structure.
 
         Parameters
@@ -278,7 +266,9 @@ class GradedMesher(Mesher):
         return np.array(min_steps)
 
     @staticmethod
-    def rotate_structure_bounds(structures: List[Structure], axis: Axis) -> List[Array[float]]:
+    def rotate_structure_bounds(
+        structures: List[Structure], axis: Axis
+    ) -> List[ArrayLike[float, 1]]:
         """Get sturcture bounding boxes with a given ``axis`` rotated to z.
 
         Parameters
@@ -290,7 +280,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        List[Array[float]]
+        List[ArrayLike[float, 1]]
             A list of the bounding boxes of shape ``(2, 3)`` for each structure, with the bounds
             along ``axis`` being ``(:, 2)``.
         """
@@ -305,70 +295,71 @@ class GradedMesher(Mesher):
         return struct_bbox
 
     @staticmethod
-    def bounds_2d_tree(struct_bbox: List[Array[float]]):
+    def bounds_2d_tree(struct_bbox: List[ArrayLike[float, 1]]):
         """Make a shapely Rtree for the 2D bounding boxes of all structures in the plane
         perpendicular to the meshing axis."""
 
         boxes_2d = []
-        for str_ind, bbox in enumerate(struct_bbox):
-            box = IndexedBox(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
-            box.str_ind = str_ind
+        for bbox in struct_bbox:
+            box = shapely_box(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
             boxes_2d.append(box)
         return STRtree(boxes_2d)
 
     @staticmethod
-    def contained_2d(bbox0: Array[float], query_bbox: List[Array[float]]) -> List[Array[float]]:
+    def contained_2d(
+        bbox0: ArrayLike[float, 1], query_bbox: List[ArrayLike[float, 1]]
+    ) -> List[ArrayLike[float, 1]]:
         """Return a list of all bounding boxes among ``query_bbox`` that contain ``bbox0`` in 2D."""
-        contained_in = []
-        for bbox in query_bbox:
+        return [
+            bbox
+            for bbox in query_bbox
             if all(
                 [
-                    bbox0[0, 0] >= bbox[0, 0],
-                    bbox0[1, 0] <= bbox[1, 0],
-                    bbox0[0, 1] >= bbox[0, 1],
-                    bbox0[1, 1] <= bbox[1, 1],
+                    bbox0[0, 0] + fp_eps >= bbox[0, 0],
+                    bbox0[1, 0] <= bbox[1, 0] + fp_eps,
+                    bbox0[0, 1] + fp_eps >= bbox[0, 1],
+                    bbox0[1, 1] <= bbox[1, 1] + fp_eps,
                 ]
-            ):
-                contained_in.append(bbox)
-        return contained_in
+            )
+        ]
 
     @staticmethod
-    def contains_3d(bbox0: Array[float], query_bbox: List[Array[float]]) -> List[int]:
+    def contains_3d(bbox0: ArrayLike[float, 1], query_bbox: List[ArrayLike[float, 1]]) -> List[int]:
         """Return a list of all indexes of bounding boxes in the ``query_bbox`` list that ``bbox0``
         fully contains."""
-        contains = []
-        for ind, bbox in enumerate(query_bbox):
+        return [
+            ind
+            for ind, bbox in enumerate(query_bbox)
             if all(
                 [
-                    bbox[0, 0] >= bbox0[0, 0],
-                    bbox[1, 0] <= bbox0[1, 0],
-                    bbox[0, 1] >= bbox0[0, 1],
-                    bbox[1, 1] <= bbox0[1, 1],
-                    bbox[0, 2] >= bbox0[0, 2],
-                    bbox[1, 2] <= bbox0[1, 2],
+                    bbox[0, 0] + fp_eps >= bbox0[0, 0],
+                    bbox[1, 0] <= bbox0[1, 0] + fp_eps,
+                    bbox[0, 1] + fp_eps >= bbox0[0, 1],
+                    bbox[1, 1] <= bbox0[1, 1] + fp_eps,
+                    bbox[0, 2] + fp_eps >= bbox0[0, 2],
+                    bbox[1, 2] <= bbox0[1, 2] + fp_eps,
                 ]
-            ):
-                contains.append(ind)
-        return contains
+            )
+        ]
 
     @staticmethod
     def is_close(coord: float, interval_coords: List[float], coord_ind: int, atol: float) -> bool:
         """Check if a given ``coord`` is within ``atol`` of an interval coordinate at a given
         interval index. If the index is out of bounds, return ``False``."""
-        is_close = False
-        if 0 <= coord_ind < len(interval_coords):
-            is_close = isclose(coord, interval_coords[coord_ind], abs_tol=atol)
-        return is_close
+        return (
+            isclose(coord, interval_coords[coord_ind], abs_tol=atol)
+            if 0 <= coord_ind < len(interval_coords)
+            else False
+        )
 
     @staticmethod
-    def is_contained(normal_pos: float, contained_2d: List[Array[float]]) -> bool:
+    def is_contained(normal_pos: float, contained_2d: List[ArrayLike[float, 1]]) -> bool:
         """Check if a given ``normal_pos`` along the meshing direction is contained inside any
         of the bounding boxes that are in the ``contained_2d`` list.
         """
-        for contain_box in contained_2d:
-            if contain_box[0, 2] <= normal_pos <= contain_box[1, 2]:
-                return True
-        return False
+        return any(
+            contain_box[0, 2] <= normal_pos <= contain_box[1, 2] for contain_box in contained_2d
+        )
 
     @staticmethod
     def filter_min_step(
@@ -391,20 +382,20 @@ class GradedMesher(Mesher):
 
     def make_grid_multiple_intervals(  # pylint:disable=too-many-locals
         self,
-        max_dl_list: Array[float],
-        len_interval_list: Array[float],
+        max_dl_list: ArrayLike[float, 1],
+        len_interval_list: ArrayLike[float, 1],
         max_scale: float,
         is_periodic: bool,
-    ) -> List[Array[float]]:
+    ) -> List[ArrayLike[float, 1]]:
         """Create grid steps in multiple connecting intervals of length specified by
         ``len_interval_list``. The maximal allowed step size in each interval is given by
         ``max_dl_list``. The maximum ratio between neighboring steps is bounded by ``max_scale``.
 
         Parameters
         ----------
-        max_dl_list : Array[float]
+        max_dl_list : ArrayLike[float, 1]
             Maximal allowed step size of each interval.
-        len_interval_list : Array[float]
+        len_interval_list : ArrayLike[float, 1]
             A list of interval lengths
         max_scale : float
             Maximal ratio between consecutive steps.
@@ -413,7 +404,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        List[Array[float]]
+        List[ArrayLike[float, 1]]
             A list of of step sizes in each interval.
         """
 
@@ -430,17 +421,16 @@ class GradedMesher(Mesher):
         )
 
         # initialize grid steps
-        dl_list = []
-        for interval_ind in range(num_intervals):
-            dl_list.append(
-                self.make_grid_in_interval(
-                    left_dl_list[interval_ind],
-                    right_dl_list[interval_ind],
-                    max_dl_list[interval_ind],
-                    max_scale,
-                    len_interval_list[interval_ind],
-                )
+        dl_list = [
+            self.make_grid_in_interval(
+                left_dl_list[interval_ind],
+                right_dl_list[interval_ind],
+                max_dl_list[interval_ind],
+                max_scale,
+                len_interval_list[interval_ind],
             )
+            for interval_ind in range(num_intervals)
+        ]
 
         # refinement
         refine_edge = 1
@@ -488,19 +478,19 @@ class GradedMesher(Mesher):
 
     def grid_multiple_interval_analy_refinement(
         self,
-        max_dl_list: Array[float],
-        len_interval_list: Array[float],
+        max_dl_list: ArrayLike[float, 1],
+        len_interval_list: ArrayLike[float, 1],
         max_scale: float,
         is_periodic: bool,
-    ) -> Tuple[Array[float], Array[float]]:
+    ) -> Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]:
         """Analytical refinement for multiple intervals. "analytical" meaning we allow
         non-integar step sizes, so that we don't consider snapping here.
 
         Parameters
         ----------
-        max_dl_list : Array[float]
+        max_dl_list : ArrayLike[float, 1]
             Maximal allowed step size of each interval.
-        len_interval_list : Array[float]
+        len_interval_list : ArrayLike[float, 1]
             A list of interval lengths
         max_scale : float
             Maximal ratio between consecutive steps.
@@ -509,7 +499,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        Tuple[Array[float], Array[float]]
+        Tuple[ArrayLike[float, 1], ArrayLike[float, 1]]
             left and right step sizes of each interval.
         """
 
@@ -577,7 +567,7 @@ class GradedMesher(Mesher):
         max_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> Array[float]:
+    ) -> ArrayLike[float, 1]:
         """Create a set of grid steps in an interval of length ``len_interval``,
         with first step no larger than ``max_scale * left_neighbor_dl`` and last step no larger than
         ``max_scale * right_neighbor_dl``, with maximum ratio ``max_scale`` between
@@ -598,7 +588,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        Array[float]
+        ArrayLike[float, 1]
             A list of step sizes in the interval.
         """
 
@@ -683,7 +673,7 @@ class GradedMesher(Mesher):
         max_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> Array[float]:
+    ) -> ArrayLike[float, 1]:
         """In an interval, grid grows, plateau, and decrease, resembling Lambda letter but
         with plateau in the connection part..
 
@@ -752,7 +742,7 @@ class GradedMesher(Mesher):
         right_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> Array[float]:
+    ) -> ArrayLike[float, 1]:
         """In an interval, grid grows, and decrease, resembling Lambda letter.
 
         Parameters
@@ -840,7 +830,7 @@ class GradedMesher(Mesher):
         large_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> Array[float]:
+    ) -> ArrayLike[float, 1]:
         """In an interval, grid grows, then plateau.
 
         Parameters
@@ -856,7 +846,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        Array[float]
+        ArrayLike[float, 1]
             A list of step sizes in the interval, in ascending order.
         """
         # steps for scaling
@@ -894,7 +884,7 @@ class GradedMesher(Mesher):
         small_dl: float,
         max_scale: float,
         len_interval: float,
-    ) -> Array[float]:
+    ) -> ArrayLike[float, 1]:
         """Mesh simply grows in an interval.
 
         Parameters
@@ -908,7 +898,7 @@ class GradedMesher(Mesher):
 
         Returns
         -------
-        Array[float]
+        ArrayLike[float, 1]
             A list of step sizes in the interval, in ascending order.
         """
 
@@ -959,13 +949,17 @@ class GradedMesher(Mesher):
                 )
 
             # solve for new scaling factor
-            sol_scale = root_scalar(fun_scale, bracket=[1, max_scale])
-            # if not converged, let's use the last strategy
-            if sol_scale.converged:
-                new_scale = sol_scale.root
+            # let's not raise exception here, but manually check the convergence.
+            root_scalar = Brentq(raise_on_fail=False, epsilon=_ROOTS_TOL)
+            sol_scale = root_scalar(fun_scale, 1, max_scale)
+
+            # convergence check based on pyroots API and manual evaluation of the function.
+            if sol_scale.converged and abs(fun_scale(sol_scale.x0)) <= _ROOTS_TOL:
+                new_scale = sol_scale.x0
                 dl_list = np.array([small_dl * new_scale**i for i in range(num_step)])
                 dl_list = np.append(small_dl, dl_list)
                 return dl_list
+            # if not converged, let's use the strategy below.
 
         # nothing more we can do, let's just add smallest step size,
         # and scale each pixel
