@@ -2,16 +2,17 @@
 """Defines properties of the medium / materials"""
 
 from abc import ABC, abstractmethod
-from typing import List, Tuple, Union, Callable
+from typing import Tuple, Union, Callable
 
 import pydantic as pd
 import numpy as np
 
-from .base import Tidy3dBaseModel
+from .base import Tidy3dBaseModel, cached_property
 from .types import PoleAndResidue, Ax, FreqBound
 from .viz import add_ax_if_none
 from .validators import validate_name_str
-from ..constants import C_0, pec_val, EPSILON_0, HERTZ, CONDUCTIVITY, PERMITTIVITY, RADPERSEC
+from ..constants import C_0, pec_val, EPSILON_0
+from ..constants import HERTZ, CONDUCTIVITY, PERMITTIVITY, RADPERSEC, MICROMETER, SECOND
 from ..log import log, ValidationError
 
 # evaluate frequency as this number (Hz) if inf
@@ -59,7 +60,7 @@ class AbstractMedium(ABC, Tidy3dBaseModel):
         None,
         title="Frequency Range",
         description="Optional range of validity for the medium.",
-        units=HERTZ,
+        units=(HERTZ, HERTZ),
     )
 
     _name_validator = validate_name_str()
@@ -79,6 +80,21 @@ class AbstractMedium(ABC, Tidy3dBaseModel):
             Complex-valued relative permittivity evaluated at ``frequency``.
         """
 
+    def nk_model(self, frequency: float) -> Tuple[float, float]:
+        """Real and imaginary parts of the refactive index as a function of frequency.
+
+        Parameters
+        ----------
+        frequency : float
+            Frequency to evaluate permittivity at (Hz).
+        Returns
+        -------
+        Tuple[float, float]
+            Real part (n) and imaginary part (k) of refractive index of medium.
+        """
+        eps_complex = self.eps_model(frequency=frequency)
+        return self.eps_complex_to_nk(eps_complex)
+
     @ensure_freq_in_range
     def eps_diagonal(self, frequency: float) -> Tuple[complex, complex, complex]:
         """Main diagonal of the complex-valued permittivity tensor as a function of frequency.
@@ -368,7 +384,7 @@ class AnisotropicMedium(AbstractMedium):
 class DispersiveMedium(AbstractMedium, ABC):
     """A Medium with dispersion (propagation characteristics depend on frequency)"""
 
-    @property
+    @cached_property
     @abstractmethod
     def pole_residue(self):
         """Representation of Medium as a pole-residue model."""
@@ -409,13 +425,14 @@ class PoleResidue(DispersiveMedium):
         1.0,
         title="Epsilon at Infinity",
         description="Relative permittivity at infinite frequency (:math:`\\epsilon_\\infty`).",
+        units=PERMITTIVITY,
     )
 
-    poles: List[PoleAndResidue] = pd.Field(
-        [],
+    poles: Tuple[PoleAndResidue, ...] = pd.Field(
+        (),
         title="Poles",
-        description="List of complex-valued (:math:`a_i, c_i`) poles for the model.",
-        units=RADPERSEC,
+        description="Tuple of complex-valued (:math:`a_i, c_i`) poles for the model.",
+        units=(RADPERSEC, RADPERSEC),
     )
 
     @ensure_freq_in_range
@@ -431,7 +448,7 @@ class PoleResidue(DispersiveMedium):
             eps -= c_cc / (1j * omega + a_cc)
         return eps
 
-    @property
+    @cached_property
     def pole_residue(self):
         """Representation of Medium as a pole-residue model."""
 
@@ -467,9 +484,10 @@ class Sellmeier(DispersiveMedium):
     >>> eps = sellmeier_medium.eps_model(200e12)
     """
 
-    coeffs: List[Tuple[float, pd.PositiveFloat]] = pd.Field(
+    coeffs: Tuple[Tuple[float, pd.PositiveFloat], ...] = pd.Field(
         title="Coefficients",
-        description="List of Sellmeier (:math:`B_i, C_i`) coefficients (unitless, microns^2).",
+        description="List of Sellmeier (:math:`B_i, C_i`) coefficients.",
+        units=(None, MICROMETER + "^2"),
     )
 
     def _n_model(self, frequency: float) -> complex:
@@ -489,7 +507,7 @@ class Sellmeier(DispersiveMedium):
         n = self._n_model(frequency)
         return AbstractMedium.nk_to_eps_complex(n)
 
-    @property
+    @cached_property
     def pole_residue(self):
         """Representation of Medium as a pole-residue model."""
 
@@ -563,12 +581,14 @@ class Lorentz(DispersiveMedium):
         1.0,
         title="Epsilon at Infinity",
         description="Relative permittivity at infinite frequency (:math:`\\epsilon_\\infty`).",
+        units=PERMITTIVITY,
     )
 
-    coeffs: List[Tuple[float, float, float]] = pd.Field(
+    coeffs: Tuple[Tuple[float, float, float], ...] = pd.Field(
         ...,
         title="Epsilon at Infinity",
-        description="List of (:math:`\\Delta\\epsilon_i, f_i, \\delta_i`) values for model (Hz).",
+        description="List of (:math:`\\Delta\\epsilon_i, f_i, \\delta_i`) values for model.",
+        units=(PERMITTIVITY, HERTZ, HERTZ),
     )
 
     @ensure_freq_in_range
@@ -580,7 +600,7 @@ class Lorentz(DispersiveMedium):
             eps += (de * f**2) / (f**2 - 2j * frequency * delta - frequency**2)
         return eps
 
-    @property
+    @cached_property
     def pole_residue(self):
         """Representation of Medium as a pole-residue model."""
 
@@ -596,8 +616,7 @@ class Lorentz(DispersiveMedium):
                 c0 = de * w**2 / 4 / r
                 a1 = -d - r
                 c1 = -c0
-                poles.append((a0, c0))
-                poles.append((a1, c1))
+                poles.extend(((a0, c0), (a1, c1)))
             else:
                 r = np.sqrt(w * w - d * d)
                 a = -d - 1j * r
@@ -633,12 +652,14 @@ class Drude(DispersiveMedium):
         1.0,
         title="Epsilon at Infinity",
         description="Relative permittivity at infinite frequency (:math:`\\epsilon_\\infty`).",
+        units=PERMITTIVITY,
     )
 
-    coeffs: List[Tuple[float, pd.PositiveFloat]] = pd.Field(
+    coeffs: Tuple[Tuple[float, pd.PositiveFloat], ...] = pd.Field(
         ...,
         title="Coefficients",
-        description="List of (:math:`f_i, \\delta_i`) values for model (Hz).",
+        description="List of (:math:`f_i, \\delta_i`) values for model.",
+        units=(HERTZ, HERTZ),
     )
 
     @ensure_freq_in_range
@@ -650,25 +671,23 @@ class Drude(DispersiveMedium):
             eps -= (f**2) / (frequency**2 + 1j * frequency * delta)
         return eps
 
-    @property
+    @cached_property
     def pole_residue(self):
         """Representation of Medium as a pole-residue model."""
 
         poles = []
+        a0 = 0j
+
         for (f, delta) in self.coeffs:
 
             w = 2 * np.pi * f
             d = 2 * np.pi * delta
 
             c0 = (w**2) / 2 / d + 0j
-            a0 = 0j
-
             c1 = -c0
             a1 = -d + 0j
 
-            poles.append((a0, c0))
-            poles.append((a1, c1))
-
+            poles.extend(((a0, c0), (a1, c1)))
         return PoleResidue(
             eps_inf=self.eps_inf,
             poles=poles,
@@ -698,12 +717,14 @@ class Debye(DispersiveMedium):
         1.0,
         title="Epsilon at Infinity",
         description="Relative permittivity at infinite frequency (:math:`\\epsilon_\\infty`).",
+        units=PERMITTIVITY,
     )
 
-    coeffs: List[Tuple[float, pd.PositiveFloat]] = pd.Field(
+    coeffs: Tuple[Tuple[float, pd.PositiveFloat], ...] = pd.Field(
         ...,
         title="Coefficients",
-        description="List of (:math:`\\Delta\\epsilon_i, \\tau_i`) values for model (Hz, sec).",
+        description="List of (:math:`\\Delta\\epsilon_i, \\tau_i`) values for model.",
+        units=(PERMITTIVITY, SECOND),
     )
 
     @ensure_freq_in_range
@@ -715,7 +736,7 @@ class Debye(DispersiveMedium):
             eps += de / (1 - 1j * frequency * tau)
         return eps
 
-    @property
+    @cached_property
     def pole_residue(self):
         """Representation of Medium as a pole-residue model."""
 
