@@ -1,17 +1,98 @@
 """ Defines 'types' that various fields can be """
 
-from typing import Tuple, List, Union
+from typing import Tuple, Union
 
 # Literal only available in python 3.8 + so try import otherwise use extensions
 try:
     from typing import Literal
 except ImportError:
     from typing_extensions import Literal
+from typing_extensions import Annotated
 
 import pydantic
 import numpy as np
 from matplotlib.axes._subplots import Axes
 from shapely.geometry.base import BaseGeometry
+from ..log import ValidationError
+
+# import warnings
+# warnings.filterwarnings(action="error", category=np.ComplexWarning)
+""" Numpy Arrays """
+
+# type tag default name
+TYPE_TAG_STR = "type"
+
+
+def annotate_type(UnionType):  # pylint:disable=invalid-name
+    """Annotated union type using TYPE_TAG_STR as discriminator."""
+    return Annotated[UnionType, pydantic.Field(discriminator=TYPE_TAG_STR)]
+
+
+# generic numpy array
+Numpy = np.ndarray
+
+
+class TypedArrayLike(np.ndarray):
+    """A numpy array with a type given by cls.inner_type"""
+
+    @classmethod
+    def make_tuple(cls, v):
+        """Converts a nested list of lists into a list of tuples."""
+        return (
+            tuple(cls.make_tuple(x) for x in v)
+            if isinstance(v, list)
+            else cls.inner_type(v)  # pylint:disable=no-member
+        )
+
+    @classmethod
+    def __get_validators__(cls):
+        """boilerplate"""
+        yield cls.validate_type
+
+    @classmethod
+    def validate_type(cls, val):
+        """validator"""
+        # need to fix, doesnt work for simulationdata_export and load?
+
+        if isinstance(val, np.ndarray):
+            val_ndims = len(val.shape)
+            cls_ndims = cls.ndims  # pylint:disable=no-member
+
+            if (cls_ndims is not None) and (cls_ndims != val_ndims):
+                raise ValidationError(
+                    "wrong number of dimensions given. " f"Given {val_ndims}, expected {cls_ndims}."
+                )
+            return cls.make_tuple(val.tolist())
+
+        return tuple(val)
+
+    @classmethod
+    def __modify_schema__(cls, field_schema):
+        """Sets the schema of ArrayLike."""
+
+        field_schema.update(
+            dict(
+                title="Array Like",
+                description="Accepts sequence (tuple, list, numpy array) and converts to tuple.",
+                type="tuple",
+                properties={},
+                required=[],
+            )
+        )
+
+
+class ArrayLikeMeta(type):
+    """metclass for Array, enables Array[type] -> TypedArray"""
+
+    def __getitem__(cls, type_ndims):
+        """Array[type, ndims] -> TypedArrayLike"""
+        desired_type, ndims = type_ndims
+        return type("Array", (TypedArrayLike,), {"inner_type": desired_type, "ndims": ndims})
+
+
+class ArrayLike(np.ndarray, metaclass=ArrayLikeMeta):
+    """type of numpy array with annotated type (Array[float], Array[complex])"""
+
 
 """ Complex Values """
 
@@ -33,7 +114,7 @@ class tidycomplex(complex):  # pylint: disable=invalid-name
 
     @classmethod
     def __get_validators__(cls):
-        """Defines which validator function to use for NumpyArray."""
+        """Defines which validator function to use for ComplexNumber."""
         yield cls.validate
 
     @classmethod
@@ -49,7 +130,7 @@ class tidycomplex(complex):  # pylint: disable=invalid-name
 
     @classmethod
     def __modify_schema__(cls, field_schema):
-        """Sets the schema of NumpyArray."""
+        """Sets the schema of ComplexNumber."""
         field_schema.update(ComplexNumber.schema())
 
 
@@ -60,11 +141,11 @@ Size = Tuple[Size1D, Size1D, Size1D]
 Coordinate = Tuple[float, float, float]
 Coordinate2D = Tuple[float, float]
 Bound = Tuple[Coordinate, Coordinate]
-GridSize = Union[pydantic.PositiveFloat, List[pydantic.PositiveFloat]]
+GridSize = Union[pydantic.PositiveFloat, Tuple[pydantic.PositiveFloat, ...]]
 Axis = Literal[0, 1, 2]
 Axis2D = Literal[0, 1]
-Vertices = List[Coordinate2D]
 Shapely = BaseGeometry
+Vertices = Union[Tuple[Coordinate2D, ...], ArrayLike[float, 2]]
 
 """ medium """
 
@@ -86,104 +167,10 @@ Direction = Literal["+", "-"]
 
 EMField = Literal["Ex", "Ey", "Ez", "Hx", "Hy", "Hz"]
 FieldType = Literal["Ex", "Ey", "Ez", "Hx", "Hy", "Hz"]
+FreqArray = Union[Tuple[float, ...], ArrayLike[float, 1]]
+ObsGridArray = Union[Tuple[float, ...], ArrayLike[float, 1]]
+RadVec = Literal["Ntheta", "Nphi", "Ltheta", "Lphi"]
 
 """ plotting """
 
 Ax = Axes
-
-""" Numpy """
-
-# generic numpy array
-Numpy = np.ndarray
-
-
-class TypedArray(np.ndarray):
-    """A numpy array with a type given by cls.inner_type"""
-
-    @classmethod
-    def __get_validators__(cls):
-        """boilerplate"""
-        yield cls.validate_type
-
-    @classmethod
-    def validate_type(cls, val):
-        """validator"""
-        # need to fix, doesnt work for simulationdata_export and load?
-        return np.array(val, dtype=cls.inner_type)  # pylint: disable=no-member
-
-    @classmethod
-    def __modify_schema__(cls, field_schema):
-        """Sets the schema of NumpyArray."""
-        field_schema.update(NumpyArray.schema())
-
-
-class ArrayMeta(type):
-    """metclass for Array, enables Array[type] -> TypedArray"""
-
-    def __getitem__(cls, t):
-        """Array[t] -> TypedArray"""
-        return type("Array", (TypedArray,), {"inner_type": t})
-
-
-class Array(np.ndarray, metaclass=ArrayMeta):
-    """type of numpy array with annotated type (Array[float], Array[complex])"""
-
-
-""" note:
-    ^ this is the best way to declare numpy types if you know dtype.
-    for example: ``field_amps: Array[float] = np.random.random(5)``.
-"""
-
-
-class NumpyArray(pydantic.BaseModel):
-    """Wrapper around numpy arrays that has a well defined json schema."""
-
-    data_list: List
-
-    @property
-    def arr(self):
-        """Contructs a numpy array representation of the NumpyArray."""
-        return np.array(self.data_list)
-
-
-class tidynumpy(np.ndarray):  # pylint: disable=invalid-name
-    """Numpy array type that we can use in place of np.ndarray."""
-
-    @classmethod
-    def __get_validators__(cls):
-        """Defines which validator function to use for NumpyArray."""
-        yield cls.validate
-
-    @classmethod
-    def validate(cls, value):
-        """What gets called when you construct a tidynumpy."""
-        if isinstance(value, NumpyArray):
-            return value.arr
-        if isinstance(value, dict):
-            numpy_array = NumpyArray(**value)
-            return numpy_array.arr
-        if isinstance(value, list):
-            return value
-        return np.array(value)
-
-    @classmethod
-    def __modify_schema__(cls, field_schema):
-        """Sets the schema of NumpyArray."""
-        field_schema.update(NumpyArray.schema())
-
-
-ArrayLike = Union[tidynumpy, NumpyArray, List]
-
-
-# lists or np.ndarrays of certain type
-FloatArrayLike = Union[List[float], Array[float]]
-IntArrayLike = Union[List[int], Array[int]]
-ComplexArrayLike = Union[List[complex], Array[complex]]
-
-# encoding for JSON in tidy3d data
-# technically not used yet since the tidy3d data has separate load and export methods.
-def numpy_encoding(array):
-    """json encoding of a (maybe complex-valued) numpy array."""
-    if array.dtype == "complex":
-        return {"re": list(array.real), "im": list(array.imag)}
-    return list(array)
