@@ -11,6 +11,12 @@ from .derivatives import create_d_matrices as d_mats
 from .derivatives import create_s_matrices as s_mats
 from .transforms import radial_transform, angled_transform
 
+# Consider vec to be complex if norm(vec.imag)/norm(vec) > TOL_COMPLEX
+TOL_COMPLEX = fp_eps
+# Tolerance for eigs
+TOL_EIGS = fp_eps
+# Tolerance for deciding on the matrix to be diagonal or tensorial
+TOL_TENSORIAL = 1e-6
 
 # pylint:disable=too-many-statements,too-many-branches,too-many-locals
 def compute_modes(
@@ -50,21 +56,17 @@ def compute_modes(
     omega = 2 * np.pi * freq
     k0 = omega / C_0
 
-    try:
-        if isinstance(eps_cross, Numpy):
-            eps_xx, eps_yy, eps_zz = eps_cross
-        elif len(eps_cross) == 3:
-            eps_xx, eps_yy, eps_zz = [np.copy(e) for e in eps_cross]
-        else:
-            raise ValueError
-    except Exception as e:
-        print("Wrong input to mode solver pemittivity!")
-        raise e
+    if isinstance(eps_cross, Numpy):
+        eps_xx, eps_yy, eps_zz = eps_cross
+    elif len(eps_cross) == 3:
+        eps_xx, eps_yy, eps_zz = [np.copy(e) for e in eps_cross]
+    else:
+        raise ValueError("Wrong input to mode solver pemittivity!")
 
     Nx, Ny = eps_xx.shape
     N = eps_xx.size
 
-    if coords[0].size != Nx + 1 or coords[1].size != Ny + 1:
+    if len(coords[0]) != Nx + 1 or len(coords[1]) != Ny + 1:
         raise ValueError("Mismatch between 'coords' and 'esp_cross' shapes.")
     new_coords = [np.copy(c) for c in coords]
 
@@ -114,27 +116,11 @@ def compute_modes(
     mu_tensor = np.einsum("ij...,pj...->ip...", mu_tensor, jac_h)
     mu_tensor /= jac_h_det
 
-    """ The forward derivative matrices already impose PEC boundary at the xmax and ymax interfaces.
-    Here, we also impose PEC boundaries on the xmin and ymin interfaces through the permittivity at
-    those positions, unless a PMC symmetry is specifically requested. The PMC symmetry is
-    imposed by modifying the backward derivative matrices."""
-    dmin_pmc = [False, False]
-    if symmetry[0] != 1:
-        # PEC at the xmin edge
-        eps_tensor[1, 1, :Ny] = pec_val
-        eps_tensor[2, 2, :Ny] = pec_val
-    else:
-        # Modify the backwards x derivative
-        dmin_pmc[0] = True
-
-    if Ny > 1:
-        if symmetry[1] != 1:
-            # PEC at the ymin edge
-            eps_tensor[0, 0, ::Ny] = pec_val
-            eps_tensor[2, 2, ::Ny] = pec_val
-        else:
-            # Modify the backwards y derivative
-            dmin_pmc[1] = True
+    """ Boundaries are imposed through the derivative matrices. The forward derivative matrices
+    always impose PEC boundary at the xmax and ymax interfaces, and on the xmin and ymin
+    interfaces unless PMC symmetry is present. If so, the PMC boundary is imposed through the
+    backward derivative matrices."""
+    dmin_pmc = [sym == 1 for sym in symmetry]
 
     # Primal grid steps for E-field derivatives
     dl_f = [new_cs[1:] - new_cs[:-1] for new_cs in new_coords]
@@ -142,7 +128,7 @@ def compute_modes(
     dl_tmp = [(dl[:-1] + dl[1:]) / 2 for dl in dl_f]
     dl_b = [np.hstack((d1[0], d2)) for d1, d2 in zip(dl_f, dl_tmp)]
 
-    # Derivative matrices with PEC boundaries at the far end and optional pmc at the near end
+    # Derivative matrices with PEC boundaries by default and optional PMC at the near end
     der_mats_tmp = d_mats((Nx, Ny), dl_f, dl_b, dmin_pmc)
 
     # PML matrices; do not impose PML on the bottom when symmetry present
@@ -160,18 +146,27 @@ def compute_modes(
         target = n_max
     else:
         target = mode_spec.target_neff
-    target_neff_p = target / np.linalg.norm(kp_to_k)
+    target_neff_p = target / np.linalg.norm(kp_to_k) + fp_eps
 
     # Solve for the modes
-    E, H, neff, keff = solver_em(eps_tensor, mu_tensor, der_mats, num_modes, target_neff_p)
+    E, H, neff, keff = solver_em(
+        Nx,
+        Ny,
+        eps_tensor,
+        mu_tensor,
+        der_mats,
+        num_modes,
+        target_neff_p,
+        mode_spec.precision,
+    )
 
-    # Reorder if needed
-    if mode_spec.sort_by != "largest_neff":
-        if mode_spec.sort_by == "te_fraction":
-            sort_int = np.sum(np.abs(E[0]) ** 2, axis=0) / np.sum(np.abs(E[:2]) ** 2, axis=(0, 1))
-        elif mode_spec.sort_by == "tm_fraction":
-            sort_int = np.sum(np.abs(E[1]) ** 2, axis=0) / np.sum(np.abs(E[:2]) ** 2, axis=(0, 1))
-        sort_inds = np.argsort(sort_int)[::-1]
+    # Filter polarization if needed
+    if mode_spec.filter_pol is not None:
+        te_int = np.sum(np.abs(E[0]) ** 2, axis=0) / np.sum(np.abs(E[:2]) ** 2, axis=(0, 1))
+        if mode_spec.filter_pol == "te":
+            sort_inds = np.concatenate((np.nonzero(te_int >= 0.5)[0], np.nonzero(te_int < 0.5)[0]))
+        elif mode_spec.filter_pol == "tm":
+            sort_inds = np.concatenate((np.nonzero(te_int <= 0.5)[0], np.nonzero(te_int > 0.5)[0]))
         E = E[..., sort_inds]
         H = H[..., sort_inds]
         neff = neff[..., sort_inds]
@@ -189,12 +184,18 @@ def compute_modes(
     return fields, neff + 1j * keff
 
 
-def solver_em(eps_tensor, mu_tensor, der_mats, num_modes, neff_guess):
+def solver_em(
+    Nx, Ny, eps_tensor, mu_tensor, der_mats, num_modes, neff_guess, mat_precision
+):  # pylint:disable=too-many-arguments
     """Solve for the electromagnetic modes of a system defined by in-plane permittivity and
     permeability and assuming translational invariance in the normal direction.
 
     Parameters
     ----------
+    Nx : int
+        Number of grids along x-direction.
+    Ny : int
+        Number of grids along y-direction.
     eps_tensor : np.ndarray
         Shape (3, 3, N), the permittivity tensor at every point in the plane.
     mu_tensor : np.ndarray
@@ -205,6 +206,8 @@ def solver_em(eps_tensor, mu_tensor, der_mats, num_modes, neff_guess):
         Number of modes to solve for.
     neff_guess : float
         Initial guess for the effective index.
+    mat_precision : Union['single', 'double']
+        Single or double-point precision in eigensolver.
 
     Returns
     -------
@@ -218,16 +221,56 @@ def solver_em(eps_tensor, mu_tensor, der_mats, num_modes, neff_guess):
         Imaginary part of the effective index, shape (num_modes, ).
     """
 
+    # Determine if ``eps`` and ``mu`` are diagonal or tensorial
     off_diagonals = (np.ones((3, 3)) - np.eye(3)).astype(bool)
     eps_offd = np.abs(eps_tensor[off_diagonals])
     mu_offd = np.abs(mu_tensor[off_diagonals])
-    if np.any(eps_offd > 1e-6) or np.any(mu_offd > 1e-6):
-        return solver_tensorial(eps_tensor, mu_tensor, der_mats, num_modes, neff_guess)
+    is_tensorial = False
+    if np.any(eps_offd > TOL_TENSORIAL) or np.any(mu_offd > TOL_TENSORIAL):
+        is_tensorial = True
 
-    return solver_diagonal(eps_tensor, mu_tensor, der_mats, num_modes, neff_guess)
+    # Determine data types
+    mat_dtype = np.float32
+    # In tensorial case, even though the matrix can be real, the
+    # expected eigenvalue is purely imaginary. So for now we enforce
+    # the matrix to be complex type so that it will look for the right eigenvalues.
+    if is_tensorial:
+        mat_dtype = np.complex128 if mat_precision == "double" else np.complex64
+    else:
+        # 1) check if complex or not
+        complex_solver = False
+        if (
+            isinstance_complex(eps_tensor)
+            or isinstance_complex(mu_tensor)
+            or np.any([isinstance_complex(f) for f in der_mats])
+        ):
+            complex_solver = True
+        # 2) determine precision
+        if complex_solver:
+            mat_dtype = np.complex128 if mat_precision == "double" else np.complex64
+        else:
+            if mat_precision == "double":
+                mat_dtype = np.float64
+
+    # data type conversion
+    eps_tensor = type_conversion(eps_tensor, mat_dtype)
+    mu_tensor = type_conversion(mu_tensor, mat_dtype)
+    der_mats = [type_conversion(f, mat_dtype) for f in der_mats]
+    neff_guess = type_conversion(np.array([neff_guess]), mat_dtype)[0]
+
+    # initial vector for eigensolver
+    vec_init = set_initial_vec(Nx, Ny, mat_dtype=mat_dtype, is_tensorial=is_tensorial)
+
+    # call solver
+    if is_tensorial:
+        return solver_tensorial(eps_tensor, mu_tensor, der_mats, num_modes, vec_init, neff_guess)
+
+    return solver_diagonal(eps_tensor, mu_tensor, der_mats, num_modes, vec_init, neff_guess)
 
 
-def solver_diagonal(eps, mu, der_mats, num_modes, neff_guess):
+def solver_diagonal(
+    eps, mu, der_mats, num_modes, vec_init, neff_guess
+):  # pylint:disable=too-many-arguments
     """EM eigenmode solver assuming ``eps`` and ``mu`` are diagonal everywhere."""
 
     N = eps.shape[-1]
@@ -258,7 +301,7 @@ def solver_diagonal(eps, mu, der_mats, num_modes, neff_guess):
     mat = pmat.dot(qmat)
 
     # Call the eigensolver. The eigenvalues are -(neff + 1j * keff)**2
-    vals, vecs = solver_eigs(mat, num_modes, guess_value=-(neff_guess**2))
+    vals, vecs = solver_eigs(mat, num_modes, vec_init, guess_value=-(neff_guess**2))
     if vals.size == 0:
         raise RuntimeError("Could not find any eigenmodes for this waveguide")
     vre, vim = -np.real(vals), -np.imag(vals)
@@ -294,7 +337,9 @@ def solver_diagonal(eps, mu, der_mats, num_modes, neff_guess):
     return E, H, neff, keff
 
 
-def solver_tensorial(eps, mu, der_mats, num_modes, neff_guess):
+def solver_tensorial(
+    eps, mu, der_mats, num_modes, vec_init, neff_guess
+):  # pylint:disable=too-many-arguments
     """EM eigenmode solver assuming ``eps`` or ``mu`` have off-diagonal elements."""
 
     N = eps.shape[-1]
@@ -362,7 +407,7 @@ def solver_tensorial(eps, mu, der_mats, num_modes, neff_guess):
     )
 
     # Call the eigensolver. The eigenvalues are 1j * (neff + 1j * keff)
-    vals, vecs = solver_eigs(mat, num_modes, guess_value=1j * neff_guess)
+    vals, vecs = solver_eigs(mat, num_modes, vec_init, guess_value=1j * neff_guess)
     if vals.size == 0:
         raise RuntimeError("Could not find any eigenmodes for this waveguide")
     # Real and imaginary part of the effective index
@@ -397,7 +442,7 @@ def solver_tensorial(eps, mu, der_mats, num_modes, neff_guess):
     return E, H, neff, keff
 
 
-def solver_eigs(mat, num_modes, guess_value=1.0):
+def solver_eigs(mat, num_modes, vec_init, guess_value=1.0):
     """Find ``num_modes`` eigenmodes of ``mat`` cloest to ``guess_value``.
 
     Parameters
@@ -409,5 +454,86 @@ def solver_eigs(mat, num_modes, guess_value=1.0):
     guess_value : float, optional
     """
 
-    values, vectors = spl.eigs(mat, k=num_modes, sigma=guess_value, tol=fp_eps)
+    values, vectors = spl.eigs(mat, k=num_modes, sigma=guess_value, tol=TOL_EIGS, v0=vec_init)
     return values, vectors
+
+
+def isinstance_complex(vec_or_mat, tol=TOL_COMPLEX):
+    """Check if a numpy array or scipy csr_matrix has complex component by looking at
+    norm(x.imag)/norm(x)>TOL_COMPLEX
+
+    Parameters
+    ----------
+    vec_or_mat : Union[np.ndarray, sp.csr_matrix]
+    """
+
+    if isinstance(vec_or_mat, np.ndarray):
+        return np.linalg.norm(vec_or_mat.imag) / (np.linalg.norm(vec_or_mat) + fp_eps) > tol
+    if isinstance(vec_or_mat, sp.csr_matrix):
+        return spl.norm(vec_or_mat.imag) / (spl.norm(vec_or_mat) + fp_eps) > tol
+
+    raise RuntimeError("Variable type should be either numpy array or scipy csr_matrix.")
+
+
+def type_conversion(vec_or_mat, new_dtype):
+    """Convert vec_or_mat to new_type.
+
+    Parameters
+    ----------
+    vec_or_mat : Union[np.ndarray, sp.csr_matrix]
+        vec or mat to be converted.
+    new_dtype : Union[np.complex128, np.complex64, np.float64, np.float32]
+        Final type of vec or mat
+
+    Returns
+    -------
+    converted_vec_or_mat : Union[np.ndarray, sp.csr_matrix]
+    """
+
+    if new_dtype in {np.complex128, np.complex64}:
+        return vec_or_mat.astype(new_dtype)
+    if new_dtype in {np.float64, np.float32}:
+        converted_vec_or_mat = vec_or_mat.real
+        return converted_vec_or_mat.astype(new_dtype)
+
+    raise RuntimeError("Unsupported new_type.")
+
+
+def set_initial_vec(Nx, Ny, mat_dtype=np.complex128, is_tensorial=False):
+    """Set initial vector for eigs:
+    1) The field at x=0 and y=0 boundaries are set to 0. This should be
+    the case for PEC boundaries, but wouldn't hurt for non-PEC boundary;
+    2) The vector is np.complex128 by default, and will be converted to
+    approriate type afterwards.
+
+    Parameters
+    ----------
+    Nx : int
+        Number of grids along x-direction.
+    Ny : int
+        Number of grids along y-direction.
+    mat_dtype : Union[np.complex128, np.complex64, np.float64, np.float32]
+        The type of returned vector.
+    is_tensorial : bool
+        diagonal or tensorial eigenvalue problem.
+    """
+
+    # The size of the vector is len_multiplier * Nx * Ny
+    len_multiplier = 2
+    if is_tensorial:
+        len_multiplier *= 2
+
+    # Initialize the vector
+    size = (Nx, Ny, len_multiplier)
+    vec_init = np.random.random(size) + 1j * np.random.random(size)
+    vec_init = type_conversion(vec_init, mat_dtype)
+
+    # Set values at the boundary to be 0
+    if Nx > 1:
+        vec_init[0, :, :] = 0
+    if Ny > 1:
+        vec_init[:, 0, :] = 0
+
+    # Concatenate the vector appropriately
+    vec_init = np.vstack(vec_init)
+    return vec_init.flatten("F")
