@@ -5,7 +5,8 @@ import logging
 import os
 import time
 from datetime import datetime, timedelta
-from typing import List, Dict, Optional
+from typing import List, Dict
+import tempfile
 
 import requests
 from dateutil import parser
@@ -14,7 +15,7 @@ from rich.progress import Progress
 
 from . import httputils as http
 from .config import DEFAULT_CONFIG
-from .s3utils import get_s3_user, DownloadProgress
+from .s3utils import upload_file, upload_string, get_s3_sts_token, download_file
 from .task import TaskId, TaskInfo, Folder
 from ..components.data import SimulationData
 from ..components.simulation import Simulation
@@ -24,6 +25,9 @@ from ..version import __version__
 
 REFRESH_TIME = 0.3
 TOTAL_DOTS = 3
+# file names when uploading to S3
+SIM_FILE_NAME = "simulation.json"
+DATA_FILE_NAME = "extra_data.hdf5"
 
 
 def run(  # pylint:disable=too-many-arguments
@@ -32,7 +36,6 @@ def run(  # pylint:disable=too-many-arguments
     folder_name: str = "default",
     path: str = "simulation_data.hdf5",
     callback_url: str = None,
-    normalize_index: Optional[int] = 0,
 ) -> SimulationData:
     """Submits a :class:`.Simulation` to server, starts running, monitors progress, downloads,
     and loads results as a :class:`.SimulationData` object.
@@ -50,11 +53,6 @@ def run(  # pylint:disable=too-many-arguments
     callback_url : str = None
         Http PUT url to receive simulation finish event. The body content is a json file with
         fields ``{'id', 'status', 'name', 'workUnit', 'solverVersion'}``.
-    normalize_index : int = 0
-        If specified, normalizes the frequency-domain data by the amplitude spectrum of the source
-        corresponding to ``simulation.sources[normalize_index]``.
-        This occurs when the data is loaded into a :class:`.SimulationData` object.
-        To turn off normalization, set ``normalize_index`` to ``None``.
 
     Returns
     -------
@@ -69,7 +67,7 @@ def run(  # pylint:disable=too-many-arguments
     )
     start(task_id)
     monitor(task_id)
-    return load(task_id=task_id, path=path, normalize_index=normalize_index)
+    return load(task_id=task_id, path=path)
 
 
 def upload(  # pylint:disable=too-many-locals,too-many-arguments
@@ -101,11 +99,11 @@ def upload(  # pylint:disable=too-many-locals,too-many-arguments
 
     simulation.validate_pre_upload()
 
-    json_string = simulation._json_string()  # pylint:disable=protected-access
     data = {
         "taskName": task_name,
         "callbackUrl": callback_url,
     }
+
     folder = _query_or_create_folder(folder_name)
     method = f"tidy3d/projects/{folder.projectId}/tasks"
 
@@ -121,16 +119,18 @@ def upload(  # pylint:disable=too-many-locals,too-many-arguments
 
     # upload the file to s3
     log.debug("Uploading the json file")
-    client, bucket, user_id = get_s3_user()
-    key = f"users/{user_id}/{task_id}/simulation.json"
-    log.debug(f"json = {json_string}")
-    resp = client.put_object(
-        Body=json_string,
-        Bucket=bucket,
-        Key=key,
-    )
-    if resp.get("ResponseMetadata") and resp.get("ResponseMetadata").get("HTTPStatusCode") != 200:
-        raise WebError("File upload to S3 failed, please check your network or try it later.")
+
+    # pylint:disable=consider-using-with
+    data_file = tempfile.NamedTemporaryFile()
+    # data_file will be reopened and closed in _json_string
+    data_file.close()
+    # pylint:disable=protected-access
+    json_string = simulation._json_string(data_file=data_file.name)
+    if data_file.name in json_string:
+        # Upload the extra data file if needed
+        json_string.replace(data_file.name, DATA_FILE_NAME)
+        upload_file(task_id, data_file.name, DATA_FILE_NAME)
+    upload_string(task_id, json_string, SIM_FILE_NAME)
 
     # log the url for the task in the web UI
     log.debug(f"{DEFAULT_CONFIG.website_endpoint}/folders/{folder.projectId}/tasks/{task_id}")
@@ -205,9 +205,9 @@ def get_run_info(task_id: TaskId):
         Is ``None`` if run info not available.
     """
     try:
-        client, bucket, user_id = get_s3_user()
-        key = f"users/{user_id}/{task_id}/output/solver_progress.csv"
-        progress = client.get_object(Bucket=bucket, Key=key)["Body"]
+        token = get_s3_sts_token(task_id, "output/solver_progress.csv")
+        client = token.get_client()
+        progress = client.get_object(Bucket=token.get_bucket(), Key=token.get_s3_key())["Body"]
         progress_string = progress.read().split(b"\n")
         perc_done, field_decay = progress_string[-2].split(b",")
         return float(perc_done), float(field_decay)
@@ -321,30 +321,30 @@ def download(task_id: TaskId, path: str = "simulation_data.hdf5") -> None:
     """
 
     # TODO: it should be possible to load "diverged" simulations
-    _download_file(task_id, fname="monitor_data.hdf5", path=path)
+    _download_file(task_id, fname="output/monitor_data.hdf5", path=path)
 
 
-def download_json(task_id: TaskId, path: str = "simulation.json") -> None:
+def download_json(task_id: TaskId, path: str = SIM_FILE_NAME) -> None:
     """Download the `.json` file associated with the :class:`.Simulation` of a given task.
 
     Parameters
     ----------
     task_id : str
         Unique identifier of task on server.  Returned by :meth:`upload`.
-    path : str = "simulation.json"
+    path : str = SIM_FILE_NAME
         Download path to .json file of simulation (including filename).
     """
-    _download_file(task_id, fname="simulation.json", path=path)
+    _download_file(task_id, fname=SIM_FILE_NAME, path=path)
 
 
-def load_simulation(task_id: TaskId, path: str = "simulation.json") -> Simulation:
+def load_simulation(task_id: TaskId, path: str = SIM_FILE_NAME) -> Simulation:
     """Download the `.json` file of a task and load the associated :class:`.Simulation`.
 
     Parameters
     ----------
     task_id : str
         Unique identifier of task on server.  Returned by :meth:`upload`.
-    path : str = "simulation.json"
+    path : str = SIM_FILE_NAME
         Download path to .json file of simulation (including filename).
 
     Returns
@@ -370,14 +370,13 @@ def download_log(task_id: TaskId, path: str = "tidy3d.log") -> None:
     ----
     To load downloaded results into data, call :meth:`load` with option `replace_existing=False`.
     """
-    _download_file(task_id, fname="tidy3d.log", path=path)
+    _download_file(task_id, fname="output/tidy3d.log", path=path)
 
 
 def load(
     task_id: TaskId,
     path: str = "simulation_data.hdf5",
     replace_existing: bool = True,
-    normalize_index: Optional[int] = 0,
 ) -> SimulationData:
     """Download and Load simultion results into :class:`.SimulationData` object.
 
@@ -389,11 +388,6 @@ def load(
         Download path to .hdf5 data file (including filename).
     replace_existing: bool = True
         Downloads the data even if path exists (overwriting the existing).
-    normalize_index : int = 0
-        If specified, normalizes the frequency-domain data by the amplitude spectrum of the source
-        corresponding to ``simulation.sources[normalize_index]``.
-        This occurs when the data is loaded into a :class:`.SimulationData` object.
-        To turn off normalization, set ``normalize_index`` to ``None``.
 
     Returns
     -------
@@ -405,7 +399,7 @@ def load(
         download(task_id=task_id, path=path)
 
     log.info(f"loading SimulationData from {path}")
-    sim_data = SimulationData.from_file(path, normalize_index=normalize_index)
+    sim_data = SimulationData.from_file(path)
 
     final_decay_value = sim_data.final_decay_value
     shutoff_value = sim_data.simulation.shutoff
@@ -510,11 +504,10 @@ def get_tasks(
     if num_tasks is None or num_tasks > len(sort_inds):
         num_tasks = len(sort_inds)
 
-    out_dict = []
-    for ipr in range(num_tasks):
-        out_dict.append({key: item[sort_inds[ipr]] for (key, item) in store_dict.items()})
-
-    return out_dict
+    return [
+        {key: item[sort_inds[ipr]] for (key, item) in store_dict.items()}
+        for ipr in range(num_tasks)
+    ]
 
 
 def estimate_cost(task_id: str) -> float:
@@ -554,6 +547,8 @@ def _query_or_create_folder(folder_name) -> Folder:
             method = "tidy3d/projects"
             resp = http.post(method, data={"projectName": folder_name})
         folder = Folder(**resp)
+    except WebError as e:
+        raise e
     except Exception as e:  # pylint:disable=broad-except
         raise WebError("Could not create task folder") from e
 
@@ -584,28 +579,16 @@ def _download_file(task_id: TaskId, fname: str, path: str) -> None:
     log.info(f'downloading file "{fname}" to "{path}"')
 
     try:
-        client, bucket, user_id = get_s3_user()
-
-        if fname in ("monitor_data.hdf5", "tidy3d.log"):
-            key = f"users/{user_id}/{task_id}/output/{fname}"
-        else:
-            key = f"users/{user_id}/{task_id}/{fname}"
-
-        head_object = client.head_object(Bucket=bucket, Key=key)
-        size_bytes = head_object["ContentLength"]
-        with Progress() as progress:
-            download_progress = DownloadProgress(size_bytes, progress)
-            client.download_file(
-                Bucket=bucket, Filename=path, Key=key, Callback=download_progress.report
-            )
-
+        download_file(task_id, fname, path)
+    except WebError as e:
+        raise e
     except Exception as e:  # pylint:disable=broad-except
         task_info = get_info(task_id)
-        log.warning(e)
+        log.warning(str(e))
         log.error(
             "Cannot retrieve requested file, check the file name and "
             "make sure the task has run correctly. Current "
-            f"task status is '{task_info.status}.",
+            f"task status is '{task_info.status}.'",
         )
 
 
@@ -614,3 +597,17 @@ def _rm_file(path: str):
     if os.path.exists(path) and not os.path.isdir(path):
         log.debug(f"removing file {path}")
         os.remove(path)
+
+
+def _s3_grant(task_id: TaskId):
+    user = DEFAULT_CONFIG.user
+    if "expiration" in user:
+        exp = parser.parse(user["expiration"]).replace(tzinfo=None)
+        # request new temporary credential when existing one expires in 5 minutes.
+        if (exp - datetime.utcnow()).total_seconds() > 300:
+            return
+
+    method = f"tidy3d/s3upload/grant?resourceId={task_id}"
+    resp = http.get(method)
+    credentials = resp["userCredentials"]
+    DEFAULT_CONFIG.user = {**DEFAULT_CONFIG.user, **credentials}
